import java.util.BitSet;

/**
One of the possible values of a variable.

@author Michael Breen
*/
public abstract class Value extends ValueSet
{
   /** This value's identifier as defined in the model. */
   private String id;

   /** This value's index in its variable's list of values. */
   private int listIndex;

   /** The bit which corresponds to this value. */
   private BitSet valueBit = new BitSet();

   /**
   @param variable the variable of which the new
   object is to be a possible value
   @param name the new value's identifier
   */
   Value(Variable variable, String name)
   {
      super(variable);
      this.id = name;
      this.listIndex = variable.addValue(this);
      this.valueBit.set(this.listIndex);
   }

   /**
   @return a set containing only the bit
   corresponding to this value
   */
   BitSet getValueBits() { return this.valueBit; }

   /**
   Find out if this value is an element in a given
   set of values of the same variable.
   @param values a set of values of the variable
   associated with this value
   @return true if this value is contained in 'values',
   otherwise false
   @throws IllegalArgumentException if the values in the
   argument belong to a different variable to this value
   */
   public boolean in(ValueSet values)
   {
      return and(values) != null;
   }

   /**
   Get the name of this value in the model.
   Note that two values associated with different
   variables may have the same name.
   @return this value's identifier in the model
   */
   public String getName() { return this.id; }

   /**
   Get the index of this value in the list of all
   values defined for the variable.
   @return a number in the range 0 ..
   ({@link ValueSet#getVariable getVariable()}.numVals() - 1)
   */
   public int getIndex() { return this.listIndex; }
}
<#list usable_variables as var>
<@pp.changeOutputFile name="${variable_id(var.@name)}.java"/>
<@file_prologue/>
<#if var.description?size != 0>
   <@prologue_comment text=var.description indent=""/>
</#if>
public class ${variable_id(var.@name)} extends Value
{
   <@define_values variable_id(var.@name), var/>
   private ${variable_id(var.@name)}(String id)
   {
      super(Variable.${variable_id(var.@name)}, id);
   }
}
</#list>
<#--------------------  Local Macros  -------------------->

<#--
Recursively descend a value definition tree,
outputting for each leaf node a Value definition and
for branch nodes a ValueSet definition with the set
being the union of the children of that node.
Parameters:
var_id - the identifier of the variable (in the code)
node - definitions are output for this node and its
   children; a "don't care" definition is output for
   the variable node itself
-->
<#macro define_values var_id node>
   <#list node.value as child>
      <#if node.value?size != 0>
         <@define_values var_id, child/>
      </#if>
   </#list>
   <#if node.description?size != 0 && node?node_name == "value">
      <@prologue_comment node.description/>
   </#if>
   <#if node.value?size == 0>
   public static final ${var_id} ${value_id(node)} =
      new ${var_id}("<#list node?ancestors('value')?reverse as
      branch>${branch.@name}-</#list>${node.@name}");

   <#else>
      <#if node?node_name == "variable">
   /** The set of all possible values of this variable. */
   public static final ValueSet ${dont_care_id()} =<#--
-->
      <#else>
   public static final ValueSet ${value_id(node)} =<#--
-->
      </#if>
      <#list node.value as val>

      ${value_id(val)}<#if (val_index > 0)>)</#if
         ><#if val_has_next>.or(</#if
         >
      </#list>
;

   </#if>
</#macro>
